import { CheckConfig, EnterData } from './config';
import * as net from 'net';
import { AppLogger } from './app.logger';
import { promises as dns } from 'dns';

function check(host: string, port: number, timeout: number) {
  return new Promise<number>((resolve, reject) => {
    const hrstart = process.hrtime();
    const socket = new net.Socket();

    socket.connect(port, host);
    socket.setTimeout(timeout || 1000);

    socket.on('connect', () => {
      socket.destroy();
      resolve(milliseconds());
    });

    socket.on('error', (error) => {
      socket.destroy();
      reject(error);
    });

    socket.on('timeout', (error) => {
      socket.destroy();
      reject(error || 'socket TIMEOUT');
    });

    function milliseconds() {
      const hrend = process.hrtime(hrstart);
      const ms = hrend[0] * 1e3 + hrend[1] / 1e6;
      return Math.floor(ms);
    }
  });
}

export class Enter implements EnterData {
  name: string;
  address: string;
  ip: string;
  location: string;
  weight: number;
  good: boolean;
  delay: number;
  constructor(data: EnterData) {
    for (const propertyName of ['name', 'address', 'location', 'weight']) {
      this[propertyName] = data[propertyName];
    }
    this.ip = null;
    this.good = false;
    this.delay = null;
  }
  async check(checkConfig: CheckConfig) {
    try {
      this.ip = (await dns.resolve4(this.address))[0];
      this.delay = await check(
        this.address,
        checkConfig.checkPort,
        checkConfig.checkTimeout,
      );
      this.good = true;
      return null;
    } catch (e) {
      this.good = false;
      return e.toString();
    }
  }
}
